==> atmos-2.96.9.tar.gz: atmos-2.96.9/src/file_input.cpp <==
/*
    $Id: file_input.cpp 142 2010-04-03 13:50:39Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <cassert>
#include <stdexcept>
#include <sstream>
#include <iostream>

#include <ctime>

#include <boost/io/ios_state.hpp>
#include <boost/numeric/ublas/vector.hpp>

#include <boost/date_time/posix_time/posix_time.hpp>

#include "atmos.h"
#include "file_input.h"

file_input::file_input( std::istream& stm ):
	m_linenum(0),
	m_stm(stm),
	m_cached( false ),
	m_locale( m_stm.getloc(), new boost::posix_time::time_input_facet("%Y-%m-%d %H:%M:%S") ) {
}
file_input::~file_input() {
}
bool file_input::unshift() throw() {
	m_cached = true;
	return true;
}
bool file_input::shift() {
	struct linenum_guard {
			linenum_guard( size_type& linenum ): m_linenum(linenum) {}
			~linenum_guard() { m_linenum++; }
		private:
			size_type& m_linenum;
	};

	if( m_cached ) {
		m_cached = false;
		parse_line( m_linenum-1, m_line );
		return true;
	}
	
	if( std::getline( m_stm, m_line ) ) {
		linenum_guard guard( m_linenum );
		parse_line( m_linenum-1, m_line ); // can throw here
		return true;
	}
	
	return false;
}
file_input::size_type file_input::line() const throw() {
	return m_linenum;
}
//FIXME: use boost string_algo
std::string file_input::strip_spaces( const std::string& str ) {
	std::string ret(str);
	boost::trim_if( ret, boost::is_space() );
	return ret;
}
void file_input::parse_line( size_type linenum, const std::string& line ) {
	if( line.size() == 0 )
		return; // skip empty line

	std::string::value_type type = line.at(0);
	
	std::istringstream iss( line );
	iss.ignore(); // ignore type
	iss.imbue( m_locale );
	iss.exceptions(std::ios_base::badbit | std::ios_base::failbit);

	try { 
		if( type == '#' )      parse_comment_line( iss );
		else if( type == 'P' ) parse_param_line( iss );
		else if( type == 'O' ) parse_object_line( iss );
		else if( type == 'M' ) parse_mode_line( iss );
		else if( type == 'm' ) parse_measurement_line( iss );
		else if( type == 'd' ) parse_dimm_line( iss );
	} catch( const std::ios::failure& e ) {
		throw failure("Bad line format");
	}
}
void file_input::parse_comment_line( std::istringstream& stm ) {
	on_comment( stm.str() );
}
void file_input::parse_param_line( std::istringstream& stm ) {
	static time_type timestamp; // NOTE: thread unsafe
	std::string key,value;
	
	stm >> timestamp;
	std::getline(stm, key, '=');
	std::getline(stm, value);

	on_parameter(timestamp, strip_spaces(key), strip_spaces(value));
}
void file_input::parse_object_line( std::istringstream& stm ) {
	static time_type   timestamp; // NOTE: thread unsafe
	static object_type obj;       // NOTE: thread unsafe

	stm >> timestamp >> obj;

	on_object( timestamp, obj );
}
void file_input::parse_mode_line( std::istringstream& stm ) {
	static time_type timestamp; // NOTE: thread unsafe
	
	stm >> timestamp;
	
	if( stm.str().find("Background") != std::string::npos ){
		on_background( timestamp );
	} else if ( stm.str().find("Normal") != std::string::npos ) {
		on_normal( timestamp );
	} else if ( stm.str().find("Flux") != std::string::npos ) {
		on_flux( timestamp );
	} else { // unknown mode
		throw failure("Unknown mode");
	}
}
void file_input::parse_measurement_line( std::istringstream& stm ) {
	static time_type timestamp;               // NOTE: thread unsafe
	static flux_type flux( atmos::channels ); // NOTE: thread unsafe
	static vars_type vars( atmos::channels, atmos::channels ); // NOTE: thread unsafe
	static rho1_type rho1( atmos::channels, atmos::channels ); // NOTE: thread unsafe
	static rho2_type rho2( atmos::channels ); // NOTE: thread unsafe
	
	stm >> timestamp >>
	flux(0) >> flux(1) >> flux(2) >> flux(3) >> // flux vector
	vars(0,0) >> vars(1,1) >> vars(2,2) >> vars(3,3) >> vars(0,1) >> vars(0,2) >> vars(0,3) >> vars(1,2) >> vars(1,3) >> vars(2,3) >> // vars matrix
	rho1(0,0) >> rho1(1,1) >> rho1(2,2) >> rho1(3,3) >> rho1(0,1) >> rho1(0,2) >> rho1(0,3) >> rho1(1,2) >> rho1(1,3) >> rho1(2,3) >> // rho1 matrix
	rho2(0) >> rho2(1) >> rho2(2) >> rho2(3); // rho2 vector;

	on_measurements( timestamp, flux, vars, rho1, rho2 );
}
void file_input::parse_dimm_line( std::istringstream& stm ) {
	static time_type timestamp;  // NOTE: thread unsafe
	static unsigned int framenum;
	static dimm_flux_type left_flux( 3 );  // NOTE: thread unsafe
	static dimm_flux_type right_flux( 3 ); // NOTE: thread unsafe
	static dimm_vars_type xvars( 4 );      // NOTE: thread unsafe
	static dimm_vars_type yvars( 4 );      // NOTE: thread unsafe
	static dimm_vars_type common( 4 );
	static double left_fwhm,left_elp;
	static double right_fwhm,right_elp;
	static double background_flux,background_err;
	stm >> timestamp >>
	framenum >>
	left_flux(0) >> right_flux(0) >> left_flux(1) >> right_flux(1) >> left_flux(2) >> right_flux(2) >>
	xvars(0) >> yvars(0) >> xvars(1) >> yvars(1) >> xvars(2) >> yvars(2) >> xvars(3) >> yvars(3) >>
	common(0) >> common(1) >> common(2) >> common(3) >>
	left_fwhm >> left_elp >> right_fwhm >> right_elp >>
	background_flux >> background_err;
	
	on_dimm( timestamp, framenum, left_flux, right_flux, xvars, yvars );
}

file_input::failure::failure( const std::string& what):
	std::runtime_error( what ) {
}

==> atmos-2.97.12.tar.gz: atmos-2.97.12/src/file_input.cpp <==
/*
    $Id: file_input.cpp 301 2017-04-26 07:39:45Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include <sstream>
#include <stdexcept>

#include <boost/algorithm/string.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>

#include "atmos.h"
#include "file_input.h"
#include "io.h"

namespace atmos {
namespace input {

file::file( std::istream& stm ):
	m_linenum(0),
	m_stm(stm),
	m_cached( false ),
	m_locale( m_stm.getloc(), new boost::posix_time::time_input_facet("%Y-%m-%d %H:%M:%S") ) {
}
file::~file() {
}

bool file::unshift() throw() {
	m_cached = true;
	return true;
}
bool file::shift() {
	struct linenum_guard {
			linenum_guard( size_type& linenum ): m_linenum(linenum) {}
			~linenum_guard() { m_linenum++; }
		private:
			size_type& m_linenum;
	};

	if( m_cached ) {
		m_cached = false;
		parse_line( m_linenum-1, m_line );
		return true;
	}
	
	if( std::getline( m_stm, m_line ) ) {
		linenum_guard guard( m_linenum );
		parse_line( m_linenum-1, m_line ); // can throw here
		return true;
	}
	
	return false;
}
size_type file::line() const throw() {
	return m_linenum;
}
std::string file::strip_spaces( const std::string& str ) {
	std::string ret(str);
	boost::trim_if( ret, boost::is_space() );
	return ret;
}
void file::parse_line( size_type linenum, const std::string& line ) {
	if( line.size() == 0 )
		return; // skip empty line

	std::string::value_type type = line.at(0);
	
	std::istringstream iss( line );
	iss.ignore(); // ignore type
	iss.imbue( m_locale );
	iss.exceptions(std::ios_base::badbit | std::ios_base::failbit);

	try { 
		if( type == '#' )      parse_comment_line( iss );
		else if( type == 'P' ) parse_param_line( iss );
		else if( type == 'O' ) parse_object_line( iss );
		else if( type == 'M' ) parse_mode_line( iss );
		else if( type == 'm' ) parse_measurement_line( iss );
		else if( type == 'd' ) parse_dimm_line( iss );
	} catch( const std::ios::failure& e ) {
		throw failure("Bad line format");
	}
}
void file::parse_comment_line( std::istringstream& stm ) {
	on_comment( stm.str() );
}
void file::parse_param_line( std::istringstream& stm ) {
	static time_type timestamp; // NOTE: thread unsafe
	std::string key,value;
	
	stm >> timestamp;
	std::getline(stm, key, '=');
	std::getline(stm, value);

	on_parameter(timestamp, strip_spaces(key), strip_spaces(value));
}
void file::parse_object_line( std::istringstream& stm ) {
	static time_type   timestamp; // NOTE: thread unsafe
	static object_type obj;       // NOTE: thread unsafe

	stm >> timestamp >> obj;

	on_object( timestamp, obj );
}
void file::parse_mode_line( std::istringstream& stm ) {
	static time_type timestamp; // NOTE: thread unsafe
	
	stm >> timestamp;
	
	if( stm.str().find("Background") != std::string::npos ){
		on_background( timestamp );
	} else if ( stm.str().find("Normal") != std::string::npos ) {
		on_normal( timestamp );
	} else if ( stm.str().find("Flux") != std::string::npos ) {
		on_flux( timestamp );
	} else { // unknown mode
		on_unknown_mode( timestamp );
		throw failure("Unknown mode");
	}
}
void file::parse_measurement_line( std::istringstream& stm ) {
	static time_type timestamp;               // NOTE: thread unsafe
	static flux_type flux( atmos::channels ); // NOTE: thread unsafe
	static vars_type vars( atmos::channels, atmos::channels ); // NOTE: thread unsafe
	static rho1_type rho1( atmos::channels, atmos::channels ); // NOTE: thread unsafe
	static rho2_type rho2( atmos::channels ); // NOTE: thread unsafe
	
	stm >> timestamp;
	for( flux_type::size_type i = 0; i < flux.size(); ++i ) { stm >> flux(i); } // flux vector
	for( vars_type::size_type i = 0; i < BOOST_UBLAS_SAME(vars.size1(),vars.size2()); ++i ) { stm >> vars(i,i); } // variances
	for( vars_type::size_type i = 0; i < vars.size1() - 1; ++i ) {
		for( vars_type::size_type j = i + 1; j < vars.size2(); ++j ) {	stm >> vars(i,j); } // variances
	} 
	for( rho1_type::size_type i = 0; i < BOOST_UBLAS_SAME(rho1.size1(),rho1.size2()); ++i ) { stm >> rho1(i,i); } // rho1
	for( rho1_type::size_type i = 0; i < rho1.size1() - 1; ++i ) {
		for( rho1_type::size_type j = i + 1; j < rho1.size2(); ++j ) { stm >> rho1(i,j); } // rho1
	}
	for( rho2_type::size_type i = 0; i < rho2.size(); ++i ) { stm >> rho2(i); } // rho2

	on_measurements( timestamp, flux, vars, rho1, rho2 );
}
void file::parse_dimm_line( std::istringstream& stm ) {
	static time_type timestamp;  // NOTE: thread unsafe
	static unsigned int framenum;
	static dimm_flux_type left_flux( 3 );  // NOTE: thread unsafe
	static dimm_flux_type right_flux( 3 ); // NOTE: thread unsafe
	static dimm_vars_type xvars( 4 );      // NOTE: thread unsafe
	static dimm_vars_type yvars( 4 );      // NOTE: thread unsafe
	static dimm_vars_type common( 4 );
	static double left_fwhm,left_elp;
	static double right_fwhm,right_elp;
	static double background_flux,background_err;
	stm >> timestamp >> framenum;
	for( dimm_flux_type::size_type i = 0; i < BOOST_UBLAS_SAME(left_flux.size(),right_flux.size()); ++i ) {
		stm >> left_flux(i) >> right_flux(i);
	}
	for( dimm_vars_type::size_type i = 0; i < BOOST_UBLAS_SAME(xvars.size(),yvars.size()); ++i ) {
		stm >> xvars(i) >> yvars(i);
	}
	for( dimm_vars_type::size_type i = 0; i < common.size(); ++i ) {
		stm >> common(i);
	}
	stm >> left_fwhm >> left_elp >> right_fwhm >> right_elp >> background_flux >> background_err;
	
	on_dimm( timestamp, framenum, left_flux, right_flux, xvars, yvars );
}

file::failure::failure( const std::string& what):
	std::runtime_error( what ) {
}

}}
